//
//  type_potts.cpp
//  dtbca_public_release
//
//  Created by Dhruv Batra on 8/25/11.
//  Copyright 2011 Dhruv Batra. All rights reserved.
//
//  Inspired by Vladimir Kolmogorov's typePotts.h in TRW-S

#include "type_potts.h"

/////////////////////////////////////////////////////////
void TypePotts::LoadEnergyFromFile(char *fname)
{
    // open file
    FILE *fp = fopen(fname,"rt");
    
    CHECK_NOT_NULL(fp,StringPrintf("Could not open file: %s",fname));
    
    // Read num_nodes num_labels num_edges
    fscanf(fp, "%d %d %d", &nnodes, &nlabels, &nedges);
    // if 0 edges, then 4-connected grid => Read dims
    if (!nedges) 
    {
        fscanf(fp, "%d %d\n",&nrows,&ncols);
        nedges = 2*nrows*ncols - nrows - ncols;
    }
    
    CHECK((nnodes>0) && (nlabels>0) && (nedges>0), "No. of nodes/edges/labels can't be negative. Are you sure this is a typePotts energy file?")
    
    sum_of_cardinalities = nlabels * nnodes;
    edgelist  = new int [nedges*2];

    // if 4-connected grid. 
    if (ncols) 
    {
        //Construct adjacency list for a 4-connected grid graph
		int ecount = 0, k;
		// Row-Major
        //horizontal edges
        for (int i=0; i!=nrows; ++i) 
            for (int j=0; j!=(ncols-1); ++j) 
			{
				k = i*ncols + j;
				edgelist[ecount++] = k;
				edgelist[ecount++] = k+1;
			}
		//vertical edges
        for (int i=0; i!=(nrows-1); ++i) 
            for (int j=0; j!=ncols; ++j) 
			{
				k = i*ncols + j;
				edgelist[ecount++] = k;
				edgelist[ecount++] = k+ncols;
			}
        
//		// Column-Major
//        //horizontal edges
//		for (int j=0; j!=(ncols-1); ++j) 
//			for (int i=0; i!=nrows; ++i) 
//			{
//				k = j*nrows + i;
//				edgelist[ecount++] = k;
//				edgelist[ecount++] = k+nrows;
//			}
//		//vertical edges
//		for (int j=0; j!=ncols; ++j) 
//			for (int i=0; i!=(nrows-1); ++i) 
//			{
//				k = j*nrows + i;
//				edgelist[ecount++] = k;
//				edgelist[ecount++] = k+1;
//			}
    }
    else 
    {
    }

    D = new double[nnodes*nlabels];
    V = new double[nedges]; // Potts
    
    //////////////////////////////////////////////////////////////////
    //Read energies
    int status;
    double tmp2;
    for (int i=0; i!=nnodes; ++i) 
    {
        //cout << i << ":\t";
        for (int l=0; l!=nlabels; ++l) 
        {
            //fscanf(fp, "%d ",&tmp2);
            status = fscanf(fp, "%lf ",&tmp2);
            CHECK(status>0, "Could not finish reading node potentials. Are you sure this is a typePotts energy file?");
            //D[l*nnodes+i] = tmp2;
            D[i*nlabels + l] = tmp2;
            
            // keep track of max entry
            if (tmp2 > DV_maxval)
                DV_maxval = tmp2;
            // keep track of min entry
            if (tmp2 < DV_minval)
                DV_minval = tmp2;
            
        }
        //cout << endl;
    }

    int n0, n1;
    if (ncols) // if 4c-grid 
    {
        for (int e=0; e != nedges; ++e) 
            V[e] = default_potts_lambda;
        
        // keep track of max entry
        if (default_potts_lambda > DV_maxval)
            DV_maxval = default_potts_lambda;
        // keep track of min entry
        if (default_potts_lambda < DV_minval)
            DV_minval = default_potts_lambda;
    }
    else 
    {
        for (int e=0; e != nedges; ++e) 
        {
		    status = fscanf(fp, "%d %d %lf", &n0, &n1, &tmp2);
            CHECK(status>0, "Could not finish reading edge potentials. Are you sure this is a typePotts energy file?");
		    //cout << n0 << "--" << n1 << "\t" << tmp2 << endl;
            
			edgelist[2*e] = n0;
			edgelist[2*e+1] = n1;
		    V[e] = tmp2;
            
            // keep track of max entry
            if (tmp2 > DV_maxval)
                DV_maxval = tmp2;
            // keep track of max entry
            if (tmp2 < DV_minval)
                DV_minval = tmp2;
        }
    }

    // close file
    fclose(fp);
}

/////////////////////////////////////////////////////////
void TypePotts::RoundEnergy()
{
    // Scale to Inttype

    // Node energies
    for (int i=0; i!=nnodes; ++i) 
        for (int l=0; l!=nlabels; ++l) 
            D[i*nlabels + l] = (Inttype) ((D[i*nlabels + l] - DV_minval) * MAX_INT/DV_maxval);
    // Edge energies
    for (int e=0; e != nedges; ++e) 
        V[e] = (Inttype) ((V[e]-DV_minval) * MAX_INT/DV_maxval);

}
