/*
 *  mplp_alg.h
 *  mplp
 *
 *  Created by Amir Globerson and David Sontag on 6/25/08.
 *  Copyright 2008 MIT. All rights reserved.
 *
 */
using namespace std;
#include <vector>
#include <iostream>
#include <map>

#include "muldim_arr.h"

class Region
{
public:
	// The variable size for each variable in the region
//	Vec m_var_sizes;
	// The size of the state-space for the region
	int m_region_prodsize;
	// The variables in the region
	vector<int> m_region_inds;
//	vector<int> m_var_sizes;	
	MulDimArr m_region_lambda;
	// Specifies the indices of the intersection sets
	vector<int> m_intersect_inds;
	// Specifies the indices corresponding to the position of each
	// intersection set in the region
	vector<vector<int> > m_inds_of_intersects;
	// Contains the messages from each region to its intersection sets
	vector<MulDimArr> m_msgs_from_region;
	vector<int> m_var_sizes; 
	
	Region(vector<int> & region_inds, MulDimArr & region_lambda, vector<vector<int> > & all_intersects, vector<int> & intersect_inds, vector<int> & var_sizes);
	double GetValFromGlobal(vector<int> & global_assignment);
	
	void UpdateMsgs(vector<MulDimArr> & sum_into_intersects);
	int Get_nVars() {return m_var_sizes.size();};
};

class MPLPAlg
{
public:
	vector<vector<int> > m_all_intersects;
	vector<Region> m_all_regions;
	vector<vector<int> > m_all_region_inds;	
	vector<MulDimArr> m_sum_into_intersects;
	vector<MulDimArr> m_all_lambdas;
	vector<vector<int> > m_all_region_intersects;
	vector<int> m_var_sizes;			
	vector<int> m_decoded_res;
	vector<double> m_objhist;
	vector<double> m_inthist;
	vector<double> m_timehist;

	// Added by Dhruv Batra for extra book-keeping
	vector<double> m_objhist_outer; // dual and primal values as a function of batches of cycles added (outer-loop iterations)
	vector<double> m_inthist_outer;
	vector<double> m_timehist_outer;
	
	// This map allows us to quickly look up the index of edge intersection sets
	map<pair<int, int>, int> m_intersect_map;

	// Added by Dhruv Batra for quick lookup for region ids of edges & nodes
	map<pair<int,int>, int> m_region_map_edge;
	vector<int> m_region_map_node; // Note this assumes that nodes are positively numbered
	
	MPLPAlg(char *regions_fname, char *intersects_fname, char *region_intersects_fname, char *lambdas_fname, char *varsizes_fname);
	void RunMPLP(int niter, double obj_del_thr, double int_gap_thr);
	
	// DB 01/24/10 -- A number of different constructors added by Dhruv Batra for ICML11 DTRW experiments 
    // To be used depending upon type of energy.
	MPLPAlg(int n_rows, int n_cols, vector<int> n_labels, vector<double> potts_lambda, char* unary_fname);
	MPLPAlg(int n_rows, int n_cols, vector<int> n_labels, vector<double> potts_lambda, double* unary_data);
    MPLPAlg(int n_nodes, int n_edges, int n_labels, int* edgelist, vector<double> potts_lambda, double* unary_data);
    MPLPAlg(int n_nodes, int n_edges, int* cardinalities, int* edgelist, double* edge_energy, double* unary_data, bool transpose=0);
    // Helper functions that the above constructors use
	void grid_structinit(int n_rows, int n_cols, vector<int> n_labels);
    void edgelist_structinit(int n_nodes, int n_edges, vector<int> n_labels, int* edgelist, bool transpose=0);
    void region_init(vector<double> potts_lambda, double* unary_data);
    void region_init(double* edge_energy, double* unary_data);

	// Add a new region and return its index. intersect_inds refers to the index of the intersection sets that this
	// region intersects with (that is, the index into m_all_intersects)
	int AddRegion(vector<int> & inds_of_vars, MulDimArr & region_lambda,  vector<int> & intersect_inds);
	// As before, but use all zero potential
	int AddRegion(vector<int> & inds_of_vars, vector<int> & intersect_inds);
	// As in the Matlab code, for now we will assume that an intersetion set is added before adding the regions that
	// intersect with it
	int AddIntersectionSet(vector<int> & inds_of_vars);
	// Find the index number into m_all_intersects of a given set of variables' intersection set.
	// Returns -1 if not found.
	int FindIntersectionSet(vector<int> & inds_of_vars);
	
	void Read(char *fname);
	// Changed by Dhruv Batra
	//void Write(char *msgs_fname,char *res_fname, char *suminto_fname, char *objhist_fname, char *inthist_fname, char *timehist_fname);
	void Write(char *msgs_fname,char *res_fname, char *suminto_fname, char *objhist_fname, char *inthist_fname, char *timehist_fname, char* objname_outer_fname = NULL, char* inthist_outer_fname = NULL, char* timehist_outer_fname = NULL, char* regint_fname = NULL);
};
